/***************************************************************************/
/*  Copyright 1993 National Instruments Corporation.  All Rights Reserved. */
/***************************************************************************/

#include <utility.h>
#include <gpib.h>
#include <formatio.h>
#include <ansi_c.h>
#include "wt95.h"

/*= Wavetek Model 95 20 MHz Synthesized Arbitrary Function Generator ======= */
/*  LabWindows 2.0 Instrument Driver                                         */
/*  Original Release: August 1992                                            */
/*  By:  EM, National Instruments Germany, Munich, Germany                   */
/*  Originally written in C                                                  */
/*  Modification History:                                                    */
/*  Modified By: DK, TU Ostrava, Czech Republic                              */
/*  Date: November 1993                                                      */
/*  Modifications:      Converted to multiple instrument driver using the    */
/*                      return value of the function for error reporting.    */
/*                                                                           */
/* Modifications:        Changed the GPIB Library calls so that they check   */
/*                       for the error bit being set to indicate an error    */
/*                       (& 0x8000) instead of checking for a negative       */
/*                       return value.                                       */
/*                                                                           */
/* Modification Date:    April 20, 1994                                      */
/*                                                                           */
/* Modified by:          JRO, National Instruments, Austin, Texas            */
/*                       Phone (800) 433-3488  Fax (512) 794-5678            */
/*                                                                           */
/* ========================================================================= */
/*          10/31/94:    Added necessary codes to create DLL                 */ 
/*                       Unreferenced local variables have been deleted      */
/*       Modified by:    SA,  LWILDP                                         */
/*                                                                           */
/* ========================================================================= */
/*                                                                           */
/* Modifications:        Modified for MDLL Compatibility                     */
/*                                                                           */
/* Modification Date:    June 18, 1996                                       */
/*                                                                           */
/* Modified by:          DS, National Instruments, Austin, Texas             */
/*                       Phone (800) 433-3488  Fax (512) 794-5678            */
/*                                                                           */                                                                          
/*****************************************************************************/ 
/*= INSTRUMENT TABLE ======================================================*/
/* address array: contains the GPIB addresses of opened instruments.       */
/* bd array: contains the device descriptors returned by OpenDev.          */
/* instr_cnt: contains the number of instruments open of this model type.  */
/*=========================================================================*/
static int address[wtk95_MAX_INSTR + 1];
static int bd[wtk95_MAX_INSTR + 1];
static int instr_cnt;

/*= STATIC VARIABLES ======================================================*/
/* cmd is a buffer for GPIB I/O strings.                                   */
/* wtk_err: the error variable for the instrument module                   */
/* ibcnt: contains the number of bytes transferred by GPIB reads and       */
/*        writes.  See the GPIB library I/O Class for more information     */
/*=========================================================================*/
static char cmd[70];
static int wtk95_err;

/*= UTILITY ROUTINES ======================================================*/
int wtk95_open_instr (int, int *);
int wtk95_close_instr (int);
int wtk95_invalid_integer_range (int, int, int, int);
int wtk95_invalid_longint_range (long, long, long, int);
int wtk95_invalid_real_range (double, double, double, int);
int wtk95_device_closed (int);
int wtk95_read_data (int, char *, int);
int wtk95_write_data (int, char *, int);
int wtk95_range_number (double, double);
double wtk95_deviation_per_volt (int);
int wtk95_valid_mod_range (double, double, double, double, double);
int wtk95_Check_Mode (int, char*, int, int);

/*=========================================================================*/
/* Function: Initialize                                                    */
/* Purpose:  This function opens the instrument, queries the instrument    */
/*           for its ID, and initializes the instrument to a known state.  */
/*=========================================================================*/
int PUBLIC wtk95_init (int addr,int id_query,int rest,int *instrID)
{
    int  ID;

    if (wtk95_invalid_integer_range (addr, 0, 30, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (id_query, 0, 1, -2) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (rest, 0, 1, -3) != 0)
        return wtk95_err;

    if (wtk95_open_instr (addr, &ID) != 0)
        return wtk95_err;

    if (id_query) {
        if (wtk95_write_data (ID, "V?", 2) != 0) {
            wtk95_close_instr (ID);
            return wtk95_err;
        }
        if (wtk95_read_data (ID, cmd, 70) != 0) {
            wtk95_close_instr (ID);
            return wtk95_err;
        }

        Scan (cmd, "WVTK 95");
        if (NumFmtdBytes () != 7) {
            wtk95_err = 223;
            wtk95_close_instr (ID);
            return wtk95_err;
        }
    }
    if (rest) {
        if (wtk95_write_data (ID, "RESET;", 1) != 0) {
            wtk95_close_instr (ID);
            return wtk95_err;
        }
    }
    *instrID = ID;
    return wtk95_err;
}

/*=========================================================================*/
/* Function: General Configuration                                         */
/* Purpose:  This function configures the basic setup of the Model 95,     */
/*           including mode waveform type, frequency, amplitude, offset,   */
/*           output, and output impedance.                                 */
/*=========================================================================*/
int PUBLIC wtk95_config (int instrID,int wvf_type,double freq,double ampl, 
                        double offset,double dcout,int output,int imped)
{
    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (wvf_type, 0, 8, -2) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (freq, 0.001, 2.0E+7, -3) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (ampl, 0.001, 15.0, -4) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (offset, -7.5, 7.5, -5) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (dcout, -20.0, 20.0, -6) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (output, 0, 1, -7) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (imped, 0, 4, -8) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    /*   Check that the amplitude and offset are within the correct range  */
    if (ampl + offset > 20.0) {
        wtk95_err = 309;
        return wtk95_err;
    }
    switch (wvf_type) {
    case 3: /*  DC output  */
        Fmt (cmd, "%s<MODE C; FU %d[b4]; DCOUT %f; OP %d[b4]; OS %d[b4]; EX", wvf_type,
                        dcout, output, imped);
        break;
    case 8: /*  Current waveform  */
        Fmt (cmd, "%s<MODE C; FR %f; AM %f; OF %f; OP %d[b4]; OS %d[b4]; EX", freq,
                        ampl, offset, output, imped);
        break;
    default: /*  Sets waveform  */
        Fmt (cmd, "%s<MODE C; FU %d[b4]; FR %f; AM %f; OF %f; OP %d[b4]; OS %d[b4]; EX",
                        wvf_type, freq, ampl, offset, output, imped);
        break;
    }
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Variable Time Symetry                                         */
/* Purpose:  This function programs the Model 95 for variable time         */
/*           symmetry mode.                                                */
/*=========================================================================*/
int PUBLIC wtk95_variable_time_symmetry (int instrID,int sym_enable,int symmetry)
{
    int func;

    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (sym_enable, 0, 1, -2) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (symmetry, 5, 95, -3) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    /*  Ensure that the Model 95 is within the correct function limits     */
    if (wtk95_write_data (instrID, "FU?", 3) != 0)
        return wtk95_err;

    if (wtk95_read_data (instrID, cmd, 50) != 0)
        return wtk95_err;
    if(Scan (cmd, "%s>%s[dt#]%d[b4]", &func) != 2) {
        wtk95_err = 236;
        return wtk95_err;
    }

    if (wtk95_invalid_integer_range (func, 0, 2,  300) != 0)
        return wtk95_err;
    if (sym_enable == 1)
        Fmt (cmd, "%s<SY %d[b4]; EX", symmetry);
    else
        Fmt (cmd, "%s<SY 50; EX");
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) !=0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Phase Lock/Phase Shift                                        */
/* Purpose:  Programs the Model 95 for phase lock and phase shift mode.    */
/*=========================================================================*/
int PUBLIC wtk95_phase_lock_shift (int instrID,int source,int phase)
{
    int mode_val;
    int symm_val;
    double freq;

    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (source, 0, 1, -2) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (phase, -180, 180, -3) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    /* Test to ensure that the Model 95 is within the limits for the Phase */
    /* Lock and Phase Shift mode.                                          */
    if (wtk95_write_data (instrID, "FR?", 3) != 0)
        return wtk95_err;

    if (wtk95_read_data (instrID, cmd, 50) != 0)
        return wtk95_err;
    if (Scan (cmd, "%s>%s[dt#]%f", &freq) != 2) {
        wtk95_err = 236;
        return wtk95_err;
    }

    if (wtk95_invalid_real_range (freq, 20.0, 2.0E+7,  301) != 0)
        return wtk95_err;

    if (wtk95_write_data (instrID, "MO?", 3) != 0)
        return wtk95_err;

    if (wtk95_read_data (instrID, cmd, 50) != 0)
        return wtk95_err;
    if (Scan (cmd, "%s>%s[dt#]%d[b4]", &mode_val) != 2) {
        wtk95_err = 236;
        return wtk95_err;
    }

    if (mode_val != 0 && mode_val != 4 && mode_val != 5) {
        wtk95_err = 302;
        return wtk95_err;
    }
    if (wtk95_write_data (instrID, "SY?", 3) != 0)
        return wtk95_err;

    if (wtk95_read_data (instrID, cmd, 50) != 0)
        return wtk95_err;
    if (Scan (cmd, "%s>%s[dt#]%d[b4]", &symm_val) != 2) {
        wtk95_err = 236;
        return wtk95_err;
    }
    if (symm_val != 50) {
        wtk95_err = 303;
        return wtk95_err;
    }
    if (!source)
        Fmt (cmd, "%s<SRC %i[b4]; EX", source);
    else
        Fmt (cmd, "%s<SRC %i[b4]; P %i[b4]; EX", source, phase);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
         return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function: Store/Recal setting                                          */
/*  Purpose:  This fuction configures the Model 95 to store and recall     */
/*            arbitrary waveform settings.                                 */
/*=========================================================================*/
int PUBLIC wtk95_store_recall_settings (int instrID,int setup,int param)
{
    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (setup, 0, 1, -2) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (param, 1, 10, -3) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    if (setup)
        Fmt (cmd, "RCL %d[b4];EX", param);
    else
        Fmt (cmd, "STS %d[b4];EX", param);

    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function: Trigger Mode                                                 */
/*  Purpose:  Configures the Model 95 for Trigger mode.                    */
/*=========================================================================*/
int PUBLIC wtk95_trigger_mode (int instrID,int source,double trig_freq)
{
    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (source, 0, 1, -2) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (trig_freq, 0.001, 2.0E+7, -3) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID ) != 0)
        return wtk95_err;

    Fmt (cmd, "%s<MODE T; SRC %i[b4]; TF %f;EX", source, trig_freq);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function: Gated Mode                                                   */
/*  Purpose:  Configures the Model 95 for gate mode.                       */
/*=========================================================================*/
int PUBLIC wtk95_gated_mode (int instrID,int source,double trig_freq)
{
    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (source, 0, 1, -2) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (trig_freq, 0.001, 2.0E+7, -3) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID ) != 0)
        return wtk95_err;

    Fmt (cmd, "%s<MODE G; SRC %i[b4]; TF %f;EX", source, trig_freq);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function: Burst Mode                                                   */
/*  Purpose:  Configures the Model 95 for burst mode.                      */
/*=========================================================================*/
int PUBLIC wtk95_burst_mode (int instrID,int source,double trig_freq,long burst_count)
{
    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (source, 0, 1, -2) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (trig_freq, 0.001, 2.0E+7, -3) != 0)
        return wtk95_err;
    if (wtk95_invalid_longint_range (burst_count, 0L, 1000000L, -4) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID ) != 0)
        return wtk95_err;

    Fmt (cmd, "%s<MODE B; B %i[b4]; SRC %i[b4]; TF %f;EX", burst_count, source,
                trig_freq);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}
/*=========================================================================*/
/*  Function: Amplidude Modulation Mode                                    */
/*  Purpose:  This funciton programs the Model 95 for Amplitude modulation.*/
/*=========================================================================*/
int PUBLIC wtk95_amplitude_modulation (int instrID)

{
    int ampl;

    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

 /*  Check to make sure that the amplitude is within the correct range */
    if (wtk95_write_data (instrID, "AM?", 3) != 0)
        return wtk95_err;

    if (wtk95_read_data (instrID, cmd, 50) != 0)
        return wtk95_err;
    if (Scan (cmd, "%s>%s[dt#]%d[b4]", &ampl) != 2) {
        wtk95_err = 236;
        return wtk95_err;
    }

    if (wtk95_invalid_real_range ((double)ampl, 0.001, 7.5, 0) != 0) {
        wtk95_err = 304;
        return wtk95_err;
    }

    /* Check to make sure that DC mode is not choosen. */
    if (wtk95_Check_Mode ( instrID, "FU?", 3, 305) != 0)
        return wtk95_err;

    if (wtk95_write_data (instrID, "MODE AM; EX", 11) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function: SCM Mode                                                     */
/*  Purpose:  This function programs the Model 95 for Suppressed Carrier   */
/*            Mode                                                         */
/*=========================================================================*/
int PUBLIC wtk95_suppressed_carrier_mod (int instrID)

{
    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    /* Check to make sure that DC mode is not choosen. */
    if (wtk95_Check_Mode ( instrID, "FU?", 3, 305) != 0)
        return wtk95_err;

    Fmt (cmd, "%s<MODE SC; EX");
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function: FM Mode                                                      */
/*  Purpose:  This function configures the Model 95 for frequency          */
/*            modulation                                                   */
/*=========================================================================*/
int PUBLIC wtk95_freq_modulation (int instrID,double cent_freq,double peak_dev,double *mod_ampl)
{
    double up_lim;
    double low_lim;
    double dev_per_volt;
    int range_number;

    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (cent_freq, 0.002, 2.0E+7, -2)!= 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (peak_dev, 0.002, 2.0E+7, -3) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    /* Check to make sure that DC mode is not choosen. */
    if (wtk95_Check_Mode ( instrID, "FU?", 3, 305) != 0)
        return wtk95_err;

    /*  Calculate the upper and lower frequency modulation limits.*/
    up_lim = cent_freq + peak_dev;
    low_lim = cent_freq - peak_dev;
    if (wtk95_invalid_real_range (up_lim, 0.002, 2.0E+7,  306) != 0)
        return wtk95_err;
    if (peak_dev > cent_freq) {
        wtk95_err = 307;
        return wtk95_err;
    }

/*  Determine the appropriate range number and corresponding deviation per */
/*  volt and the modulation amplitude given the upper and lower limits.    */

    range_number = wtk95_range_number (up_lim, low_lim);
    dev_per_volt = wtk95_deviation_per_volt (range_number);
    *mod_ampl = 2.0 * peak_dev / dev_per_volt;

    Fmt (cmd, "%s<FR %f; MODE FM; RANGELOCK ON; FR %f; EX", up_lim, cent_freq);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function: VCG Mode                                                     */
/*  Purpose:  This function configures the Model 95 for Voltage Controlled */
/*  generation of an output waveform.                                      */
/*=========================================================================*/
int PUBLIC wtk95_voltage_controlled_gen (int instrID,double cent_freq,double pos_change, 
                                    double neg_change, double *pos_Vcg,double *neg_Vcg)
{
    double up_lim;
    double low_lim;
    double dev_per_volt;
    int range_number;

    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (cent_freq, 0.002, 2.0E+7, -2) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (pos_change, 0.002, 2.0E+7, -3) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (neg_change, 0.002, 2.0E+7, -4) != 0)
        return wtk95_err;

    /*  Calculate the upper and lower frequency limits.*/
    up_lim = cent_freq + pos_change;
    low_lim = cent_freq - neg_change;
    if (wtk95_invalid_real_range (up_lim, 0.002, 2.0E+7,  306) != 0)
        return wtk95_err;

    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    /* Check to make sure that DC mode is not choosen. */
    if (wtk95_Check_Mode ( instrID, "FU?", 3, 305) != 0)
        return wtk95_err;

/*  Calculate the range number, deviation per volt, and the  corresponding */
/*  positive and negative voltage change needed for voltage controlled     */
/*  modulation.   */
    range_number = wtk95_range_number (up_lim, low_lim);
    dev_per_volt = wtk95_deviation_per_volt (range_number);
    *pos_Vcg = pos_change / dev_per_volt;
    *neg_Vcg = neg_change / dev_per_volt;

    Fmt (cmd, "%s<FR %f; MODE FM; RANGELOCK ON; FR %f; EX", up_lim, cent_freq);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function: Range Lock off                                               */
/*  Purpose:  This function turns off the Range Lock mode needed by        */
/*            waveform modulation                                          */
/*=========================================================================*/
int PUBLIC wtk95_range_lock_off (int instrID)
{

    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    if (wtk95_write_data (instrID, "RA 0", 4) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function: Sweep Mode                                                   */
/*  Purpose:  This function configures the Model 95 for sweep mode.        */
/*=========================================================================*/
int PUBLIC wtk95_sweep_mode (int instrID,int sweep_mode,int sweep_type,double start_freq,double stop_freq,
                       double sweep_rate,int trig_source,double trig_freq)
{
     if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (sweep_mode, 0, 4, -2) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (sweep_type, 0, 3, -3) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (start_freq, 0.001, 2.0E+7, -4) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (stop_freq, 0.001, 2.0E+7, -5) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (sweep_rate, 0.1, 3600.0, -6) != 0)
        return wtk95_err;
    if (wtk95_invalid_real_range (trig_freq, 0.01, 10.0, -7) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (trig_source, 0, 1, -8) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    /* Check to make sure that DC mode is not choosen. */
    if (wtk95_Check_Mode ( instrID, "FU?", 3, 305) != 0)
        return wtk95_err;

    if (sweep_mode == 3)
        if (!trig_source)
            Fmt (cmd, "%s<MO SW;STA %f;STO %f;STI %f; STY %d[b4];SMD 3;SRC %d[b4];STF %f;EX",
                    start_freq, stop_freq, sweep_rate, sweep_type,
                    trig_source, trig_freq);
        else
            Fmt (cmd, "%s<MO SW;STA %f;STO %f;STI %f; STY %d[b4];SMD 3;SRC %d[b4];EX",
                    start_freq, stop_freq, sweep_rate, sweep_type,
                    trig_source);
    else
        Fmt (cmd, "%s<MO SW;STA %f;STO %f;STI %f;STY %d[b4];SMD %d[b4];EX",
                start_freq, stop_freq, sweep_rate, sweep_type, sweep_mode);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Trigger Sweeep via GPIB                                       */
/* Purpose:  This function triggers the Model 95 via the GPIB bus.         */
/*=========================================================================*/
int PUBLIC wtk95_gpib_sweep_trigger (int instrID)
{

    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    if (wtk95_write_data (instrID, "TRG; EX", 7) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Arb Wvfm Edit Mode                                            */
/* Purpose:  This function place the Model 95 in arbitrary waveform        */
/*           Edit mode.                                                    */
/*=========================================================================*/
int PUBLIC wtk95_arb_wvfm_edit_mode (int instrID,int edit,int start_addr,
                                int stop_addr,int sync_addr)
{
    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (edit, 0, 1, -2) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (start_addr, 0, 8191, -3) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (stop_addr, 3, 8191, -4) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (sync_addr, start_addr, stop_addr, -5) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    if (start_addr > stop_addr) {
        wtk95_err = -6;
        return wtk95_err;
    }
    if (!edit)
        Fmt (cmd, "ARBEDIT; POINT; SRA %i[b4]; SPA %i[b4]; SYN %i[b4];", start_addr,
                stop_addr, sync_addr);
    else
        Fmt (cmd, "ARBEDIT; LINE; SRA %i[b4]; SPA %i[b4]; SYN %i[b4];", start_addr,
                stop_addr, sync_addr);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function: Erase Arb Waveform                                           */
/*  Purpose: This function erases an arbitrary waveform function from      */
/*           active memory.                                                */
/*=========================================================================*/
int PUBLIC wtk95_erase_arb_wvfm (int instrID)
{
    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    if (wtk95_write_data (instrID, "ABR; EX", 7) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Restore Edited Waveform                                       */
/* Purpose:  This function restores a currently stored arbitrary waveform  */
/*           in the event a mistake was made during editing.               */
/*=========================================================================*/
int PUBLIC wtk95_restore_waveform (int instrID)
{
    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    if (wtk95_write_data (instrID, "ARBEDIT RESTORE; EX", 19) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Set Arb Address                                               */
/* Purpose:  This function sets the arbitrary waveform address             */
/*           or x-axis value.                                              */
/*=========================================================================*/
int PUBLIC wtk95_set_arb_address (int instrID,int addr)
{

    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (addr, 0, 8191, -2) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    Fmt (cmd, "ADDRESS %d[b4];", addr);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Select Arb Data                                               */
/* Purpose:  This function sets an arbitrary data value.                   */
/*=========================================================================*/
int PUBLIC wtk95_set_arb_data (int instrID,int value)
{

    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (value, -2047, 2047, -2) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    Fmt (cmd, "DATA %d[b4];", value);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Filter                                                        */
/* Purpose:  This function enables the Model 95 filters.                   */
/*=========================================================================*/
int PUBLIC wtk95_filter (int instrID,int bw)
{

    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (bw, 0, 2, -2) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    Fmt (cmd, "FILTER %d[b4];", bw);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Smoothing                                                     */
/* Purpose:  This functions selects  various software equivalent           */
/*           RC filters.                                                   */
/*=========================================================================*/
int PUBLIC wtk95_smoothing (int instrID,int bw)
{

    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (bw, 1, 7, -2) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    Fmt (cmd, "SMOOTH %d[b4];", bw);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function: Store Arb Waveform                                           */
/*  Purpose:  This function stores the edited arbitrary waveform in        */
/*            active memory.                                               */
/*=========================================================================*/
int PUBLIC wtk95_arb_wvfm_store (int instrID,int wvfm)
{

    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;
    if (wtk95_invalid_integer_range (wvfm, 1, 4, -2) != 0)
        return wtk95_err;
    if (wtk95_device_closed (instrID) != 0)
        return wtk95_err;

    Fmt (cmd, "ABS %d[b4];", wvfm);
    if (wtk95_write_data (instrID, cmd, NumFmtdBytes ()) != 0)
        return wtk95_err;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Close                                                         */
/* Purpose:  This function closes the instrument.                          */
/*=========================================================================*/
int PUBLIC wtk95_close (int instrID)
{
    if (wtk95_invalid_integer_range (instrID, 1, wtk95_MAX_INSTR, -1) != 0)
        return wtk95_err;

    if (wtk95_device_closed (instrID))
        return wtk95_err;

    wtk95_close_instr (instrID);
    return wtk95_err;
}

/*= UTILITY ROUTINES ======================================================*/

/*=========================================================================*/
/* Function: Open Instrument                                               */
/* Purpose:  This function locates and initializes an entry in the         */
/*           Instrument Table and the GPIB device table for the            */
/*           instrument.  The size of the Instrument Table can be changed  */
/*           in the include file by altering the constant                  */
/*           wtk95_MAX_INSTR.  The return value of this function is equal */
/*           to the global error variable.                                 */
/*=========================================================================*/
int  wtk95_open_instr (int addr,int *ID)
{
    int i, instrID;

    instrID = 0;
    wtk95_err = 0;

/* Check to see if the instrument is already in the Instrument Table. */

    for (i = 1; i<= wtk95_MAX_INSTR; i++)
        if (address[i] == addr) {
            instrID = i;
            i = wtk95_MAX_INSTR;
         }

/* If it is not in the instrument table, open an entry for the instrument. */

    if (instrID <= 0)
        for (i = 1; i<= wtk95_MAX_INSTR; i++)
            if (address[i] == 0) {
                instrID = i;
                i = wtk95_MAX_INSTR;
             }

/* If an entry could not be opened in the Instrument Table, return an error.*/

    if (instrID <= 0) {
        wtk95_err = 220;
        return wtk95_err;
    }

/* If the device has not been opened in the GPIB device table (bd[ID] = 0),*/
/*  then open it.                                                          */

    if (bd[instrID] <= 0) {
        if (instr_cnt <= 0)
            CloseInstrDevs("wtk95");
        bd[instrID] = OpenDev ("", "wtk95");
        if (bd[instrID] <= 0) {
            wtk95_err = 220;
            return wtk95_err;
        }
        instr_cnt += 1;
        address[instrID] = addr;
     }

/*  Change the primary address of the device    */

    if (ibpad (bd[instrID], addr) & 0x8000) {
        wtk95_err = 233;
        return wtk95_err;
    }

    *ID = instrID;
    return wtk95_err;
}

/*=========================================================================*/
/* Function: Close Instrument                                              */
/* Purpose:  This function closes the instrument by removing it from the   */
/*           GPIB device table and setting the address and bd[instrID] to  */
/*           zero in the Instrument Table.  The return value is equal to   */
/*           the global error variable.                                    */
/*=========================================================================*/
int  wtk95_close_instr (int instrID)
{
    if (bd[instrID] != 0) {
        CloseDev (bd[instrID]);
        bd[instrID] = 0;
        address[instrID] = 0;
        instr_cnt -= 1;
        wtk95_err = 0;
    }
    else
        wtk95_err = 221;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int  wtk95_invalid_integer_range (int val,int min,int max,int err_code)
{
    if (val < min || val > max)
        wtk95_err = err_code;
    else
        wtk95_err = 0;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Invalid Long Integer Range                                    */
/* Purpose:  This function checks a long integer to see if it lies between */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int  wtk95_invalid_longint_range (long val,long min,long max,int err_code)
{
    if (val < min || val > max)
        wtk95_err = err_code;
    else
        wtk95_err = 0;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Invalid Real Range                                            */
/* Purpose:  This function checks a real number to see if it lies between  */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int  wtk95_invalid_real_range (double val,double min,double max,int err_code)
{
    if (val < min || val > max)
        wtk95_err = err_code;
    else
        wtk95_err = 0;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Device Closed                                                 */
/* Purpose:  This function checks to see if the module has been            */
/*           initialized.  If the device has not been opened, set the      */
/*           global error variable to 232, 0 otherwise. The return value   */
/*           is equal to the global error value.                           */
/*=========================================================================*/
int  wtk95_device_closed (int instrID)
{
    if (bd[instrID] <= 0)
        wtk95_err = 232;
    else
        wtk95_err = 0;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Read Data                                                     */
/* Purpose:  This function reads a buffer of data from the instrument. The */
/*           return value is equal to the global error variable.           */
/*=========================================================================*/
int  wtk95_read_data (int instrID,char *buf,int cnt)
{
    if (ibrd(bd[instrID], buf, (long)cnt) & 0x8000)
        wtk95_err = 231;
    else
        wtk95_err = 0;

    return wtk95_err;
}

/*=========================================================================*/
/* Function: Write Data                                                    */
/* Purpose:  This function writes a buffer of data to the instrument. The  */
/*           return value is equal to the global error variable.           */
/*=========================================================================*/
int  wtk95_write_data (int instrID,char *buf,int cnt)
{
    if (ibwrt(bd[instrID], buf, (long)cnt) & 0x8000)
        wtk95_err = 230;
    else
        wtk95_err = 0;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function:  Calculate Range Number                                      */
/*  Purpose:   This funciton calculates the range number given the upper   */
/*             and lower modulation values.Consult the User Manual         */
/*             Table 3-12.                                                 */
/*=========================================================================*/
int  wtk95_range_number (double up_val,double low_val)
{
    if (!wtk95_valid_mod_range (up_val, low_val, 0.02, 2.0, 0.002))
        return 0;
    if (!wtk95_valid_mod_range (up_val, low_val, 0.2, 20.1, 0.002))
        return 1;
    if (!wtk95_valid_mod_range (up_val, low_val, 0.201, 2.0, 0.002))
        return 2;
    if (!wtk95_valid_mod_range (up_val, low_val, 2.01, 20.0, 0.02))
        return 3;
    if (!wtk95_valid_mod_range (up_val, low_val, 20.1, 200.0, 0.2))
        return 4;
    if (!wtk95_valid_mod_range (up_val, low_val, 201.0, 2000.0, 2.0))
        return 5;
    if (!wtk95_valid_mod_range (up_val, low_val, 2.01, 20000.0, 20.0))
        return 6;
    if (!wtk95_valid_mod_range (up_val, low_val, 20.1, 2.0E+5, 200.0))
        return 7;
    if (!wtk95_valid_mod_range (up_val, low_val, 201.0, 2.0E+6, 2000.0))
        return 8;
    if (!wtk95_valid_mod_range (up_val, low_val, 2.01, 2.0E+7, 20000.0))
        return 9;

/*  Computed frequency limit is not within the proper range and modulation */
/*  limits for the Model 95                                                */

    wtk95_err = 307;

    return wtk95_err;
}

/*=========================================================================*/
/*  Function: Calculate Deviation per Volt                                 */
/*  Purpose:  This function calculated the deviation per volt given        */
/*            the range number determined by Table 3-12.                   */
/*=========================================================================*/
double wtk95_deviation_per_volt (int range_number)
{
    double dev_per_volt;

    dev_per_volt = 20.0 * pow (10.0, (double)(range_number - 4));
    return dev_per_volt;
}

/*=========================================================================*/
/*  Function: Check For Valid Modulation Range                             */
/*  Purpose:  This function checks to makes sure that the upper and lower  */
/*            modulation values are within the max and min values.         */
/*=========================================================================*/
int  wtk95_valid_mod_range (double upper_val,double lower_val,double min,double max, 
                        double lower_limit)
{

    if ((upper_val < min) || (upper_val > max) || (lower_val < lower_limit))
        return -1;
    else
        return 0;
}

/*=========================================================================*/
/*  Function: Check For Mode                                               */
/*  Purpose:  This function checks to makes sure that the specified mode   */
/*            is not choosen.                                              */
/*=========================================================================*/
int  wtk95_Check_Mode (int instrID,char *func,int val,int err_code)

{
    int mode_val;

    if (wtk95_write_data (instrID, func, StringLength (func)) != 0)
        return wtk95_err;
    if (wtk95_read_data (instrID, cmd, 70) != 0)
        return wtk95_err;
    if (Scan (cmd, "%s>%s[dt#]%d[b4]", &mode_val) != 2) {
        wtk95_err = 236;
        return wtk95_err;
    }
    if (mode_val == val) {
        wtk95_err =  err_code;
        return wtk95_err;
    }

    return wtk95_err;
}

/*=== THE END =============================================================*/
